package com.sun.org.apache.bcel.internal.util;

import com.sun.org.apache.bcel.internal.classfile.ClassParser;
import java.io.InputStream;
import java.io.IOException;
import java.util.HashMap;
import com.sun.org.apache.bcel.internal.classfile.JavaClass;
import java.lang.ref.SoftReference;
import java.util.Map;

public class SyntheticRepository implements Repository
{
    private final Map<String, SoftReference<JavaClass>> loadedClasses;
    
    private SyntheticRepository() {
        this.loadedClasses = new HashMap();
    }
    
    public static SyntheticRepository getInstance() {
        return new SyntheticRepository();
    }
    
    @Override
    public void storeClass(JavaClass clazz) {
        this.loadedClasses.put(clazz.getClassName(), new SoftReference(clazz));
        clazz.setRepository(this);
    }
    
    @Override
    public void removeClass(JavaClass clazz) {
        this.loadedClasses.remove(clazz.getClassName());
    }
    
    @Override
    public JavaClass findClass(String className) {
        SoftReference<JavaClass> ref = (SoftReference)this.loadedClasses.get(className);
        if (ref == null) {
            return null;
        }
        return (JavaClass)ref.get();
    }
    
    @Override
    public JavaClass loadClass(String className) throws ClassNotFoundException {
        if (className == null || className.isEmpty()) {
            throw new IllegalArgumentException("Invalid class name " + className);
        }
        className = className.replace('/', '.');
        JavaClass clazz = this.findClass(className);
        if (clazz != null) {
            return clazz;
        }
        IOException e = new IOException("Couldn't find: " + className + ".class");
        throw new ClassNotFoundException("Exception while looking for class " + className + ": " + e, e);
    }
    
    @Override
    public JavaClass loadClass(Class<?> clazz) throws ClassNotFoundException {
        String className = clazz.getName();
        JavaClass repositoryClass = this.findClass(className);
        if (repositoryClass != null) {
            return repositoryClass;
        }
        String name = className;
        int i = name.lastIndexOf(46);
        if (i > 0) {
            name = name.substring(i + 1);
        }
        JavaClass cls = null;
        try (final InputStream clsStream = clazz.getResourceAsStream(name + ".class")) {
            JavaClass loadClass;
            cls = (loadClass = this.loadClass(clsStream, className));
            return loadClass;
        }
        catch (IOException e) {
            return cls;
        }
    }
    
    private JavaClass loadClass(InputStream is, String className) throws ClassNotFoundException {
        try {
            if (is != null) {
                ClassParser parser = new ClassParser(is, className);
                JavaClass clazz = parser.parse();
                this.storeClass(clazz);
                return clazz;
            }
        }
        catch (IOException e) {
            throw new ClassNotFoundException("Exception while looking for class " + className + ": " + e, e);
        }
        finally {
            if (is != null) {
                try {
                    is.close();
                }
                catch (IOException ex) {}
            }
        }
        throw new ClassNotFoundException("SyntheticRepository could not load " + className);
    }
    
    @Override
    public void clear() {
        this.loadedClasses.clear();
    }
}
